@@ -68,10 +68,12 @@ SOFTWARE.
  * the y-x-banding that's so nice to have...
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gdkregion.h>
 #include "gdkregion-generic.h"
+#include "gdkalias.h"
 
 #ifdef DEBUG
 #include <stdio.h>
@@ -136,6 +138,8 @@ gdk_region_rectangle (GdkRectangle *rectangle)
 {
   GdkRegion *temp;
 
+  g_return_val_if_fail (rectangle != NULL, NULL);
+
   if (rectangle->width <= 0 || rectangle->height <= 0)
     return gdk_region_new();
 
@@ -165,6 +169,8 @@ gdk_region_copy (GdkRegion *region)
 {
   GdkRegion *temp;
 
+  g_return_val_if_fail (region != NULL, NULL);
+
   temp = g_new (GdkRegion, 1);
   temp->rects = g_new (GdkRegionBox, region->numRects);
 
@@ -180,6 +186,9 @@ gdk_region_copy (GdkRegion *region)
 void
 gdk_region_get_clipbox (GdkRegion *r, GdkRectangle *rect)
 {
+  g_return_if_fail (r != NULL);
+  g_return_if_fail (rect != NULL);
+  
   rect->x = r->extents.x1;
   rect->y = r->extents.y1;
   rect->width = r->extents.x2 - r->extents.x1;
@@ -222,12 +231,24 @@ gdk_region_get_rectangles (GdkRegion     *region,
     }
 }
 
+/**
+ * gdk_region_union_with_rect:
+ * @region: a #GdkRegion.
+ * @rect: a #GdkRectangle.
+ * 
+ * Sets the area of @region to the union of the areas of @region and
+ * @rect. The resulting area is the set of pixels contained in
+ * either @region or @rect.
+ **/
 void
 gdk_region_union_with_rect (GdkRegion    *region,
 			    GdkRectangle *rect)
 {
   GdkRegion tmp_region;
 
+  g_return_if_fail (region != NULL);
+  g_return_if_fail (rect != NULL);
+
   if (!rect->width || !rect->height)
     return;
     
@@ -306,8 +327,10 @@ miSetExtents (GdkRegion *pReg)
 void
 gdk_region_destroy (GdkRegion *r)
 {
-    g_free (r->rects);
-    g_free (r);
+  g_return_if_fail (r != NULL);
+  
+  g_free (r->rects);
+  g_free (r);
 }
 
 
@@ -324,6 +347,8 @@ gdk_region_offset (GdkRegion *region,
   int nbox;
   GdkRegionBox *pbox;
 
+  g_return_if_fail (region != NULL);
+
   pbox = region->rects;
   nbox = region->numRects;
 
@@ -405,6 +430,8 @@ gdk_region_shrink (GdkRegion *r,
   GdkRegion *s, *t;
   int grow;
 
+  g_return_if_fail (r != NULL);
+
   if (!dx && !dy)
     return;
 
@@ -513,15 +540,17 @@ miIntersectO (GdkRegion    *pReg,
  * @source1: a #GdkRegion
  * @source2: another #GdkRegion
  *
- * Converts @source1 into the intersection between @source1 and @source2.
- * That is, after calling this function @source2 will be unchanged and
- * @source1 will be the areas the two regions have in common.
- * 
+ * Sets the area of @source1 to the intersection of the areas of @source1
+ * and @source2. The resulting area is the set of pixels contained in
+ * both @source1 and @source2.
  **/
 void
 gdk_region_intersect (GdkRegion *region,
 		      GdkRegion *other)
 {
+  g_return_if_fail (region != NULL);
+  g_return_if_fail (other != NULL);
+  
   /* check for trivial reject */
   if ((!(region->numRects)) || (!(other->numRects))  ||
       (!EXTENTCHECK(&region->extents, &other->extents)))
@@ -1124,10 +1153,22 @@ miUnionO (GdkRegion *pReg,
     }
 }
 
+/**
+ * gdk_region_union:
+ * @source1:  a #GdkRegion
+ * @source2: a #GdkRegion 
+ * 
+ * Sets the area of @source1 to the union of the areas of @source1 and
+ * @source2. The resulting area is the set of pixels contained in
+ * either @source1 or @source2.
+ **/
 void
 gdk_region_union (GdkRegion *region,
 		  GdkRegion *other)
 {
+  g_return_if_fail (region != NULL);
+  g_return_if_fail (other != NULL);
+  
   /*  checks all the simple cases */
 
     /*
@@ -1379,13 +1420,16 @@ miSubtractO (GdkRegion    *pReg,
  * @source1: a #GdkRegion
  * @source2: another #GdkRegion
  *
- * Subtracts any area in @source2 from the area in @source1.
- * 
+ * Subtracts the area of @source2 from the area @source1. The resulting
+ * area is the set of pixels contained in @source1 but not in @source2.
  **/
 void
 gdk_region_subtract (GdkRegion *region,
 		     GdkRegion *other)
 {
+  g_return_if_fail (region != NULL);
+  g_return_if_fail (other != NULL);
+  
   /* check for trivial reject */
   if ((!(region->numRects)) || (!(other->numRects)) ||
       (!EXTENTCHECK(&region->extents, &other->extents)))
@@ -1408,11 +1452,9 @@ gdk_region_subtract (GdkRegion *region,
  * @source1: a #GdkRegion
  * @source2: another #GdkRegion
  *
- * XORs the two regions, placing the result in @source1.  The XOR of two
- * regions contains all areas which were not overlapping.  That is,
- * it's the union of the regions minus the intersection of the
- * regions.
- * 
+ * Sets the area of @source1 to the exclusive-OR of the areas of @source1
+ * and @source2. The resulting area is the set of pixels contained in one
+ * or the other of the two sources but not in both.
  **/
 void
 gdk_region_xor (GdkRegion *sra,
@@ -1420,6 +1462,9 @@ gdk_region_xor (GdkRegion *sra,
 {
   GdkRegion *trb;
 
+  g_return_if_fail (sra != NULL);
+  g_return_if_fail (srb != NULL);
+
   trb = gdk_region_copy (srb);
 
   gdk_region_subtract (trb, sra);
@@ -1437,6 +1482,8 @@ gdk_region_xor (GdkRegion *sra,
 gboolean
 gdk_region_empty (GdkRegion *r)
 {
+  g_return_val_if_fail (r != NULL, FALSE);
+  
   if (r->numRects == 0)
     return TRUE;
   else
@@ -1452,6 +1499,9 @@ gdk_region_equal (GdkRegion *r1,
 {
   int i;
 
+  g_return_val_if_fail (r1 != NULL, FALSE);
+  g_return_val_if_fail (r2 != NULL, FALSE);
+
   if (r1->numRects != r2->numRects) return FALSE;
   else if (r1->numRects == 0) return TRUE;
   else if (r1->extents.x1 != r2->extents.x1) return FALSE;
@@ -1476,6 +1526,8 @@ gdk_region_point_in (GdkRegion *region,
 {
   int i;
 
+  g_return_val_if_fail (region != NULL, FALSE);
+
   if (region->numRects == 0)
     return FALSE;
   if (!INBOX(region->extents, x, y))
@@ -1497,9 +1549,13 @@ gdk_region_rect_in (GdkRegion    *region,
   GdkRegionBox  rect;
   GdkRegionBox *prect = &rect;
   gboolean      partIn, partOut;
+  gint rx, ry;
 
-  gint rx = rectangle->x;
-  gint ry = rectangle->y;
+  g_return_val_if_fail (region != NULL, GDK_OVERLAP_RECTANGLE_OUT);
+  g_return_val_if_fail (rectangle != NULL, GDK_OVERLAP_RECTANGLE_OUT);
+
+  rx = rectangle->x;
+  ry = rectangle->y;
   
   prect->x1 = rx;
   prect->y1 = ry;
@@ -1645,6 +1701,9 @@ gdk_region_spans_intersect_foreach (GdkRegion  *region,
   GdkSpan *end_span;
   GdkSpan out_span;
 
+  g_return_if_fail (region != NULL);
+  g_return_if_fail (spans != NULL);
+
   if (!sorted)
     {
       gdk_region_unsorted_spans_intersect_foreach (region,
@@ -1658,10 +1717,6 @@ gdk_region_spans_intersect_foreach (GdkRegion  *region,
   if ((!region->numRects) || (n_spans == 0))
     return;
 
-  y = span->y;
-  left = span->x;
-  right = span->x + span->width; /* right is not in the span! */
-
   /* The main method here is to step along the
    * sorted rectangles and spans in lock step, and
    * clipping the spans that are in the current
@@ -1721,3 +1776,6 @@ gdk_region_spans_intersect_foreach (GdkRegion  *region,
       pbox++;
     }
 }
+
+#define __GDK_REGION_GENERIC_C__
+#include "gdkaliasdef.c"
