@@ -32,6 +32,8 @@
 #include "gtkmarshalers.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkbindings.h"
+#include "gtkprivate.h"
+#include "gtkalias.h"
 
 
 #define	MAX_DIGITS	(64)	/* don't change this,
@@ -173,42 +175,42 @@ gtk_scale_class_init (GtkScaleClass *class)
 						     -1,
 						     MAX_DIGITS,
 						     1,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_DRAW_VALUE,
-                                   g_param_spec_boolean ("draw_value",
+                                   g_param_spec_boolean ("draw-value",
 							 P_("Draw Value"),
 							 P_("Whether the current value is displayed as a string next to the slider"),
-							 FALSE,
-							 G_PARAM_READWRITE));
+							 TRUE,
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_VALUE_POS,
-                                   g_param_spec_enum ("value_pos",
+                                   g_param_spec_enum ("value-pos",
 						      P_("Value Position"),
 						      P_("The position in which the current value is displayed"),
 						      GTK_TYPE_POSITION_TYPE,
-						      GTK_POS_LEFT,
-						      G_PARAM_READWRITE));
+						      GTK_POS_TOP,
+						      GTK_PARAM_READWRITE));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("slider_length",
+					   g_param_spec_int ("slider-length",
 							     P_("Slider Length"),
 							     P_("Length of scale's slider"),
 							     0,
 							     G_MAXINT,
 							     31,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("value_spacing",
+					   g_param_spec_int ("value-spacing",
 							     P_("Value spacing"),
 							     P_("Space between value text and the slider/trough area"),
 							     0,
 							     G_MAXINT,
 							     2,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
   
   /* All bindings (even arrow keys) are on both h/v scale, because
    * blind users etc. don't care about scale orientation.
@@ -264,10 +266,10 @@ gtk_scale_class_init (GtkScaleClass *class)
   add_slider_binding (binding_set, GDK_KP_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_DOWN);
    
-  add_slider_binding (binding_set, GDK_Page_Up, 0,
+  add_slider_binding (binding_set, GDK_Page_Up, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_LEFT);
 
-  add_slider_binding (binding_set, GDK_KP_Page_Up, 0,
+  add_slider_binding (binding_set, GDK_KP_Page_Up, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_LEFT);  
 
   add_slider_binding (binding_set, GDK_Page_Up, 0,
@@ -276,10 +278,10 @@ gtk_scale_class_init (GtkScaleClass *class)
   add_slider_binding (binding_set, GDK_KP_Page_Up, 0,
                       GTK_SCROLL_PAGE_UP);
   
-  add_slider_binding (binding_set, GDK_Page_Down, 0,
+  add_slider_binding (binding_set, GDK_Page_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_RIGHT);
 
-  add_slider_binding (binding_set, GDK_KP_Page_Down, 0,
+  add_slider_binding (binding_set, GDK_KP_Page_Down, GDK_CONTROL_MASK,
                       GTK_SCROLL_PAGE_RIGHT);
 
   add_slider_binding (binding_set, GDK_Page_Down, 0,
@@ -459,7 +461,7 @@ gtk_scale_set_draw_value (GtkScale *scale,
 
       gtk_widget_queue_resize (GTK_WIDGET (scale));
 
-      g_object_notify (G_OBJECT (scale), "draw_value");
+      g_object_notify (G_OBJECT (scale), "draw-value");
     }
 }
 
@@ -485,7 +487,7 @@ gtk_scale_set_value_pos (GtkScale        *scale,
       if (GTK_WIDGET_VISIBLE (scale) && GTK_WIDGET_MAPPED (scale))
 	gtk_widget_queue_resize (GTK_WIDGET (scale));
 
-      g_object_notify (G_OBJECT (scale), "value_pos");
+      g_object_notify (G_OBJECT (scale), "value-pos");
     }
 }
 
@@ -518,7 +520,7 @@ gtk_scale_get_range_border (GtkRange  *range,
   if (scale->draw_value)
     {
       gint value_spacing;
-      gtk_widget_style_get (widget, "value_spacing", &value_spacing, NULL);
+      gtk_widget_style_get (widget, "value-spacing", &value_spacing, NULL);
 
       switch (scale->value_pos)
         {
@@ -602,7 +604,7 @@ gtk_scale_style_set (GtkWidget *widget,
   range = GTK_RANGE (widget);
   
   gtk_widget_style_get (widget,
-                        "slider_length", &slider_length,
+                        "slider-length", &slider_length,
                         NULL);
   
   range->min_slider_size = slider_length;
@@ -747,3 +749,5 @@ void _gtk_scale_clear_layout (GtkScale *scale)
     }
 }
 
+#define __GTK_SCALE_C__
+#include "gtkaliasdef.c"
